import * as tslib_1 from "tslib";
import { readdir, readFile, stat, writeFile } from 'mz/fs';
import { join, relative, resolve } from 'path';
import { collectStats, Stats } from './stats';
var CLIError = (function (_super) {
    tslib_1.__extends(CLIError, _super);
    function CLIError() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isCLIError = true;
        return _this;
    }
    return CLIError;
}(Error));
export default function run() {
    runCli()
        .catch(function (e) {
        if (e.isCLIError) {
            console.log("Error: " + e.message);
        }
        else {
            console.log(e);
        }
    });
}
function runCli() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var path, stats, coffeePaths, _a, _b, _c, i, coffeePath, source, resolvedPath, e_1_1, e_1, _d;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    if (process.argv.length !== 3) {
                        throw new CLIError('Expected exactly one CLI argument, a path to the directory to search.');
                    }
                    path = process.argv[2];
                    stats = new Stats();
                    return [4 /*yield*/, getCoffeeFiles(path)];
                case 1:
                    coffeePaths = _e.sent();
                    _e.label = 2;
                case 2:
                    _e.trys.push([2, 7, 8, 9]);
                    _a = tslib_1.__values(coffeePaths.entries()), _b = _a.next();
                    _e.label = 3;
                case 3:
                    if (!!_b.done) return [3 /*break*/, 6];
                    _c = tslib_1.__read(_b.value, 2), i = _c[0], coffeePath = _c[1];
                    process.stdout.write("\r" + (i + 1) + "/" + coffeePaths.length);
                    return [4 /*yield*/, readFile(coffeePath)];
                case 4:
                    source = (_e.sent()).toString();
                    if (source[0] === '\uFEFF') {
                        source = source.slice(1);
                    }
                    source = source.replace(/\r\n/g, '\n');
                    resolvedPath = relative('.', coffeePath);
                    try {
                        collectStats(source, stats, resolvedPath);
                    }
                    catch (e) {
                        process.stdout.write("\rError processing file " + resolvedPath + "\n");
                        if (process.env['SHOW_ERRORS'] === 'true') {
                            console.log(e);
                        }
                    }
                    _e.label = 5;
                case 5:
                    _b = _a.next();
                    return [3 /*break*/, 3];
                case 6: return [3 /*break*/, 9];
                case 7:
                    e_1_1 = _e.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 9];
                case 8:
                    try {
                        if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7 /*endfinally*/];
                case 9:
                    console.log('');
                    console.log(stats.format());
                    if (!stats.exampleContents) return [3 /*break*/, 11];
                    return [4 /*yield*/, writeFile('./soak-examples.txt', stats.exampleContents)];
                case 10:
                    _e.sent();
                    console.log('Wrote examples to soak-examples.txt');
                    _e.label = 11;
                case 11: return [2 /*return*/];
            }
        });
    });
}
function getCoffeeFiles(path) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var resultFiles, children, children_1, children_1_1, child, childPath, childStat, e_2, _a, _b, _c, e_3_1, e_3, _d;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    resultFiles = [];
                    return [4 /*yield*/, readdir(path)];
                case 1:
                    children = _e.sent();
                    _e.label = 2;
                case 2:
                    _e.trys.push([2, 12, 13, 14]);
                    children_1 = tslib_1.__values(children), children_1_1 = children_1.next();
                    _e.label = 3;
                case 3:
                    if (!!children_1_1.done) return [3 /*break*/, 11];
                    child = children_1_1.value;
                    if (['node_modules', '.git'].includes(child)) {
                        return [3 /*break*/, 10];
                    }
                    childPath = resolve(join(path, child));
                    childStat = void 0;
                    _e.label = 4;
                case 4:
                    _e.trys.push([4, 6, , 7]);
                    return [4 /*yield*/, stat(childPath)];
                case 5:
                    childStat = _e.sent();
                    return [3 /*break*/, 7];
                case 6:
                    e_2 = _e.sent();
                    return [3 /*break*/, 10];
                case 7:
                    if (!childStat.isDirectory()) return [3 /*break*/, 9];
                    _b = (_a = resultFiles.push).apply;
                    _c = [resultFiles];
                    return [4 /*yield*/, getCoffeeFiles(childPath)];
                case 8:
                    _b.apply(_a, _c.concat([tslib_1.__spread.apply(void 0, [_e.sent()])]));
                    return [3 /*break*/, 10];
                case 9:
                    if (childStat.isFile() && childPath.endsWith('.coffee')) {
                        resultFiles.push(childPath);
                    }
                    _e.label = 10;
                case 10:
                    children_1_1 = children_1.next();
                    return [3 /*break*/, 3];
                case 11: return [3 /*break*/, 14];
                case 12:
                    e_3_1 = _e.sent();
                    e_3 = { error: e_3_1 };
                    return [3 /*break*/, 14];
                case 13:
                    try {
                        if (children_1_1 && !children_1_1.done && (_d = children_1.return)) _d.call(children_1);
                    }
                    finally { if (e_3) throw e_3.error; }
                    return [7 /*endfinally*/];
                case 14: return [2 /*return*/, resultFiles];
            }
        });
    });
}
