"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lines_and_columns_1 = require("lines-and-columns");
function formatExample(node, source, path) {
    var start = node.start, end = node.end;
    start = Math.min(Math.max(start, 0), source.length);
    end = Math.min(Math.max(end, start), source.length);
    var lineMap = new lines_and_columns_1.default(source);
    var startLoc = lineMap.locationForIndex(start);
    var endLoc = lineMap.locationForIndex(end);
    if (!startLoc || !endLoc) {
        throw new Error("unable to find locations for range: [" + start + ", " + end + ")");
    }
    var displayStartLine = Math.max(0, startLoc.line - 2);
    var displayEndLine = endLoc.line + 2;
    var rows = [];
    for (var line = displayStartLine; line <= displayEndLine; line++) {
        var startOfLine = lineMap.indexForLocation({ line: line, column: 0 });
        var endOfLine = lineMap.indexForLocation({ line: line + 1, column: 0 });
        if (startOfLine === null) {
            break;
        }
        if (endOfLine === null) {
            endOfLine = source.length;
        }
        var lineSource = trimRight(source.slice(startOfLine, endOfLine));
        if (startLoc.line !== endLoc.line) {
            if (line >= startLoc.line && line <= endLoc.line) {
                rows.push([">", line + 1 + " |", lineSource]);
            }
            else {
                rows.push(["", line + 1 + " |", lineSource]);
            }
        }
        else if (line === startLoc.line) {
            var highlightLength = Math.max(endLoc.column - startLoc.column, 1);
            rows.push([">", line + 1 + " |", lineSource], ["", "|", ' '.repeat(startLoc.column) + '^'.repeat(highlightLength)]);
        }
        else {
            rows.push(["", line + 1 + " |", lineSource]);
        }
    }
    var columns = [
        { id: 'marker', align: 'right' },
        { id: 'line', align: 'right' },
        { id: 'source', align: 'left' }
    ];
    return "Example from " + path + ":\n" + printTable({ rows: rows, columns: columns }) + "\n\n";
}
exports.formatExample = formatExample;
function trimRight(string) {
    return string.replace(/\s+$/, '');
}
function printTable(table, buffer) {
    if (buffer === void 0) { buffer = ' '; }
    var widths = [];
    table.rows.forEach(function (row) {
        row.forEach(function (cell, i) {
            if (widths.length <= i) {
                widths[i] = cell.length;
            }
            else if (widths[i] < cell.length) {
                widths[i] = cell.length;
            }
        });
    });
    var output = '';
    table.rows.forEach(function (row) {
        row.forEach(function (cell, i) {
            var column = table.columns[i];
            if (column.align === 'left') {
                output += cell;
            }
            else if (column.align === 'right') {
                output += ' '.repeat(widths[i] - cell.length) + cell;
            }
            if (i < row.length - 1) {
                output += buffer;
            }
        });
        output += '\n';
    });
    return output;
}
