"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var SourceType_1 = require("coffee-lex/dist/SourceType");
var nodes_1 = require("decaffeinate-parser/dist/nodes");
function isSoakOperation(node) {
    return node instanceof nodes_1.SoakedMemberAccessOp ||
        node instanceof nodes_1.SoakedDynamicMemberAccessOp ||
        node instanceof nodes_1.SoakedFunctionApplication ||
        node instanceof nodes_1.SoakedProtoMemberAccessOp ||
        node instanceof nodes_1.SoakedNewOp ||
        node instanceof nodes_1.SoakedSlice;
}
exports.isSoakOperation = isSoakOperation;
/**
 * Determine if this is the soaked operand of a soak container.
 * In other words, the `a` in `a?.b`, `a?[b]`, or `a?(b, c)`.
 */
function isSoakedExpression(node) {
    if (node.parentNode instanceof nodes_1.SoakedMemberAccessOp &&
        node.parentNode.expression === node) {
        return true;
    }
    if (node.parentNode instanceof nodes_1.SoakedDynamicMemberAccessOp &&
        node.parentNode.expression === node) {
        return true;
    }
    if (node.parentNode instanceof nodes_1.SoakedFunctionApplication &&
        node.parentNode.function === node) {
        return true;
    }
    return false;
}
exports.isSoakedExpression = isSoakedExpression;
function findSoakContainer(node, tokens, _a) {
    var _b = (_a === void 0 ? {} : _a).ignoreParens, ignoreParens = _b === void 0 ? false : _b;
    var result = node;
    while (canParentHandleSoak(result, tokens, ignoreParens)) {
        if (!result.parentNode) {
            throw new Error('Expected parent node.');
        }
        result = result.parentNode;
    }
    return result;
}
exports.findSoakContainer = findSoakContainer;
function canParentHandleSoak(node, tokens, ignoreParens) {
    if (node.parentNode === null) {
        return false;
    }
    if (!ignoreParens && isSurroundedByParens(node, tokens)) {
        return false;
    }
    if (node.parentNode instanceof nodes_1.MemberAccessOp) {
        return true;
    }
    if (node.parentNode instanceof nodes_1.DynamicMemberAccessOp &&
        node.parentNode.expression === node) {
        return true;
    }
    if (node.parentNode instanceof nodes_1.FunctionApplication &&
        node.parentNode.function === node) {
        return true;
    }
    if (node.parentNode instanceof nodes_1.BaseAssignOp &&
        node.parentNode.assignee === node) {
        return true;
    }
    if (node.parentNode instanceof nodes_1.PostIncrementOp ||
        node.parentNode instanceof nodes_1.PostDecrementOp ||
        node.parentNode instanceof nodes_1.PreIncrementOp ||
        node.parentNode instanceof nodes_1.PreDecrementOp ||
        node.parentNode instanceof nodes_1.DeleteOp) {
        return true;
    }
    return false;
}
function isSurroundedByParens(node, tokens) {
    var prevTokenIndex = tokens.indexOfTokenNearSourceIndex(node.start - 1);
    var nextTokenIndex = tokens.indexOfTokenNearSourceIndex(node.end - 1).next();
    var prevSemanticTokenIndex = tokens.lastIndexOfTokenMatchingPredicate(isSemanticToken, prevTokenIndex);
    var nextSemanticTokenIndex = tokens.indexOfTokenMatchingPredicate(isSemanticToken, nextTokenIndex);
    if (!prevSemanticTokenIndex || !nextSemanticTokenIndex) {
        return false;
    }
    var prevToken = tokens.tokenAtIndex(prevSemanticTokenIndex);
    var nextToken = tokens.tokenAtIndex(nextSemanticTokenIndex);
    if (!prevToken || !nextToken) {
        return false;
    }
    return prevToken.type === SourceType_1.default.LPAREN && nextToken.type === SourceType_1.default.RPAREN;
}
function isSemanticToken(token) {
    return ![
        SourceType_1.default.COMMENT, SourceType_1.default.HERECOMMENT, SourceType_1.default.NEWLINE
    ].includes(token.type);
}
