"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var decaffeinate_parser_1 = require("decaffeinate-parser");
var nodes_1 = require("decaffeinate-parser/dist/nodes");
var soak_operations_1 = require("./soak-operations");
var examples_1 = require("./examples");
var Stats = (function () {
    function Stats() {
        this.totalFiles = 0;
        this.totalLines = 0;
        this.numSoakOperations = 0;
        this.numSoakedMemberAccesses = 0;
        this.numSoakedDynamicMemberAccesses = 0;
        this.numSoakedFunctionApplications = 0;
        this.numSoakedNew = 0;
        this.numNonTrivialSoakContainers = 0;
        this.numNonTrivialSoakContainersExcludingMethods = 0;
        this.numSoakedAssignments = 0;
        this.numSoakedDeletes = 0;
        this.numSignificantParens = 0;
        this.numChainedSoakOperations = 0;
        this.exampleContents = '';
    }
    Stats.prototype.format = function () {
        return "Total files: " + this.totalFiles + "\nTotal lines: " + this.totalLines + "\nTotal soak operations: " + this.numSoakOperations + "\nTotal soaked member accesses: " + this.numSoakedMemberAccesses + "\nTotal soaked dynamic member accesses: " + this.numSoakedDynamicMemberAccesses + "\nTotal soaked function applications: " + this.numSoakedFunctionApplications + "\nTotal soaked new invocations: " + this.numSoakedNew + "\nTotal soak operations using short-circuiting: " + this.numNonTrivialSoakContainers + "\nTotal soak operations using short-circuiting (excluding methods): " + this.numNonTrivialSoakContainersExcludingMethods + "\nTotal soaked assignments (including compound assignments): " + this.numSoakedAssignments + "\nTotal soaked deletes: " + this.numSoakedDeletes + "\nTotal cases where parens affected the soak container: " + this.numSignificantParens + "\nTotal soak operations chained on top of another soak: " + this.numChainedSoakOperations;
    };
    return Stats;
}());
exports.Stats = Stats;
function collectStats(source, stats, path) {
    var program = decaffeinate_parser_1.parse(source);
    var tokens = program.context.sourceTokens;
    stats.totalFiles++;
    stats.totalLines += source.split('\n').length;
    decaffeinate_parser_1.traverse(program, function (node) {
        if (node instanceof nodes_1.SoakedMemberAccessOp) {
            stats.numSoakedMemberAccesses++;
        }
        else if (node instanceof nodes_1.SoakedDynamicMemberAccessOp) {
            stats.numSoakedDynamicMemberAccesses++;
        }
        else if (node instanceof nodes_1.SoakedFunctionApplication) {
            stats.numSoakedFunctionApplications++;
        }
        else if (node instanceof nodes_1.SoakedNewOp) {
            stats.numSoakedNew++;
        }
        if (soak_operations_1.isSoakOperation(node)) {
            stats.numSoakOperations++;
            var soakContainer = soak_operations_1.findSoakContainer(node, tokens);
            var soakContainerIgnoringParens = soak_operations_1.findSoakContainer(node, tokens, { ignoreParens: true });
            if (soakContainer !== node) {
                stats.numNonTrivialSoakContainers++;
                var isMethodSoakContainer = soakContainer instanceof nodes_1.FunctionApplication &&
                    soakContainer.function === node;
                if (!isMethodSoakContainer) {
                    stats.numNonTrivialSoakContainersExcludingMethods++;
                }
            }
            if (soakContainer instanceof nodes_1.BaseAssignOp) {
                stats.numSoakedAssignments++;
            }
            if (soakContainer instanceof nodes_1.DeleteOp) {
                stats.numSoakedDeletes++;
            }
            if (soakContainer !== soakContainerIgnoringParens) {
                stats.numSignificantParens++;
            }
            if (soak_operations_1.isSoakedExpression(soakContainer)) {
                stats.numChainedSoakOperations++;
            }
            if (process.env.PRINT_EXAMPLES === 'true' && shouldShowExample(node, soakContainer)) {
                stats.exampleContents += examples_1.formatExample(node, source, path);
            }
        }
    });
}
exports.collectStats = collectStats;
function shouldShowExample(node, soakContainer) {
    if (node !== soakContainer) {
        var isMethodSoakContainer = soakContainer instanceof nodes_1.FunctionApplication &&
            soakContainer.function === node;
        return !isMethodSoakContainer;
    }
    return false;
}
