import { parse, traverse } from 'decaffeinate-parser';
import { BaseAssignOp, DeleteOp, FunctionApplication, SoakedDynamicMemberAccessOp, SoakedFunctionApplication, SoakedMemberAccessOp, SoakedNewOp, } from 'decaffeinate-parser/dist/nodes';
import { findSoakContainer, isSoakedExpression, isSoakOperation } from './soak-operations';
import { formatExample } from './examples';
var Stats = (function () {
    function Stats() {
        this.totalFiles = 0;
        this.totalLines = 0;
        this.numSoakOperations = 0;
        this.numSoakedMemberAccesses = 0;
        this.numSoakedDynamicMemberAccesses = 0;
        this.numSoakedFunctionApplications = 0;
        this.numSoakedNew = 0;
        this.numNonTrivialSoakContainers = 0;
        this.numNonTrivialSoakContainersExcludingMethods = 0;
        this.numSoakedAssignments = 0;
        this.numSoakedDeletes = 0;
        this.numSignificantParens = 0;
        this.numChainedSoakOperations = 0;
        this.exampleContents = '';
    }
    Stats.prototype.format = function () {
        return "Total files: " + this.totalFiles + "\nTotal lines: " + this.totalLines + "\nTotal soak operations: " + this.numSoakOperations + "\nTotal soaked member accesses: " + this.numSoakedMemberAccesses + "\nTotal soaked dynamic member accesses: " + this.numSoakedDynamicMemberAccesses + "\nTotal soaked function applications: " + this.numSoakedFunctionApplications + "\nTotal soaked new invocations: " + this.numSoakedNew + "\nTotal soak operations using short-circuiting: " + this.numNonTrivialSoakContainers + "\nTotal soak operations using short-circuiting (excluding methods): " + this.numNonTrivialSoakContainersExcludingMethods + "\nTotal soaked assignments (including compound assignments): " + this.numSoakedAssignments + "\nTotal soaked deletes: " + this.numSoakedDeletes + "\nTotal cases where parens affected the soak container: " + this.numSignificantParens + "\nTotal soak operations chained on top of another soak: " + this.numChainedSoakOperations;
    };
    return Stats;
}());
export { Stats };
export function collectStats(source, stats, path) {
    var program = parse(source);
    var tokens = program.context.sourceTokens;
    stats.totalFiles++;
    stats.totalLines += source.split('\n').length;
    traverse(program, function (node) {
        if (node instanceof SoakedMemberAccessOp) {
            stats.numSoakedMemberAccesses++;
        }
        else if (node instanceof SoakedDynamicMemberAccessOp) {
            stats.numSoakedDynamicMemberAccesses++;
        }
        else if (node instanceof SoakedFunctionApplication) {
            stats.numSoakedFunctionApplications++;
        }
        else if (node instanceof SoakedNewOp) {
            stats.numSoakedNew++;
        }
        if (isSoakOperation(node)) {
            stats.numSoakOperations++;
            var soakContainer = findSoakContainer(node, tokens);
            var soakContainerIgnoringParens = findSoakContainer(node, tokens, { ignoreParens: true });
            if (soakContainer !== node) {
                stats.numNonTrivialSoakContainers++;
                var isMethodSoakContainer = soakContainer instanceof FunctionApplication &&
                    soakContainer.function === node;
                if (!isMethodSoakContainer) {
                    stats.numNonTrivialSoakContainersExcludingMethods++;
                }
            }
            if (soakContainer instanceof BaseAssignOp) {
                stats.numSoakedAssignments++;
            }
            if (soakContainer instanceof DeleteOp) {
                stats.numSoakedDeletes++;
            }
            if (soakContainer !== soakContainerIgnoringParens) {
                stats.numSignificantParens++;
            }
            if (isSoakedExpression(soakContainer)) {
                stats.numChainedSoakOperations++;
            }
            if (process.env.PRINT_EXAMPLES === 'true' && shouldShowExample(node, soakContainer)) {
                stats.exampleContents += formatExample(node, source, path);
            }
        }
    });
}
function shouldShowExample(node, soakContainer) {
    if (node !== soakContainer) {
        var isMethodSoakContainer = soakContainer instanceof FunctionApplication &&
            soakContainer.function === node;
        return !isMethodSoakContainer;
    }
    return false;
}
